"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _reduxBundler = require("redux-bundler");

var _object = require("../utils/object");

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

const CREATED = 'ORG_EVENT_SOURCE_CREATED';
const REMOVED = 'ORG_EVENT_SOURCE_REMOVED';
/** @typedef {'CreateResource' | 'UpdateResource' | 'DeleteResource'} FHIRActionType */

var _default = ({
  apiUrl,
  eventFilter,
  customHandlers,
  orgIdSelector
}) => ({
  name: 'orgEventSource',
  getReducer: () => {
    const initial = {};
    return (state = initial, {
      type,
      payload
    }) => {
      if (type === CREATED) {
        return _objectSpread(_objectSpread({}, state), {}, {
          [payload.id]: payload.eventSource
        });
      }

      if (type === REMOVED) {
        return (0, _object.omit)(state, [payload]);
      }

      return state;
    };
  },
  selectOrgEventSourceRaw: state => state.orgEventSource,
  doRemoveOrgEventSource: orgId => ({
    store,
    dispatch
  }) => {
    const existing = store.selectOrgEventSourceRaw()[orgId];

    if (existing) {
      const {
        eventSource
      } = existing;

      if (eventSource) {
        try {
          eventSource.close();
        } catch (e) {// empty on purpose
        }
      }

      dispatch({
        type: REMOVED,
        payload: orgId
      });
    }
  },
  doCreateOrgEventSource: orgId => ({
    store,
    dispatch
  }) => {
    const eventSource = new EventSource(apiUrl + '/fhir-sse/event-source/' + orgId + '/' + store.selectTabId() + '?token=' + store.selectAuthToken());
    dispatch({
      type: CREATED,
      payload: {
        id: orgId,
        eventSource
      }
    });
    eventSource.addEventListener('error', () => {
      // eslint-disable-next-line
      // closed
      if (eventSource.readyState === 2) {
        setTimeout(() => {
          // if it's still 2 we should retry
          if (eventSource.readyState === 2) {
            store.doCreateOrgEventSource(orgId);
          }
        }, 1000);
      }
    });
    eventSource.addEventListener('message', message => {
      /** @type {FHIRActionType} */
      let action;
      /** @type {string} */

      let resourceType;
      /** @type {string} */

      let resourceReference;
      /** @type {string} */

      let id;

      try {
        const parsed = JSON.parse(message.data);
        action = parsed.action;
        resourceType = parsed.resourceType;
        resourceReference = parsed.data;
        id = resourceReference.split('/')[1];
      } catch (e) {
        // eslint-disable-next-line no-console
        console.log(e); // eslint-disable-next-line no-console

        console.error('Got unparsable SSE message:', message.data);
        return;
      }

      if (eventFilter && !eventFilter({
        action,
        resourceType,
        resourceReference,
        store
      })) {
        return;
      }

      const customHandler = customHandlers && customHandlers[resourceType] && customHandlers[resourceType][action];

      if (customHandler) {
        customHandler(store, {
          resourceId: id,
          orgId
        });
        return;
      } // we always fetch newly created resources
      // there's no way to know whether we need them


      if (action === 'CreateResource') {
        store.doFetchFhirResource(resourceReference, orgId);
        return;
      } // For update and delete we don't know if we have it
      // we only do something if we do


      if (action === 'UpdateResource') {
        const found = store.utilFindFhirResourceById(id);

        if (found) {
          store.doFetchFhirResource(resourceReference, orgId);
        }

        return;
      }

      if (action === 'DeleteResource') {
        const found = store.utilFindFhirResourceById(id);

        if (found) {
          store.doRemoveFhirResource({
            id,
            orgId,
            resourceType
          });
        }

        return;
      }
    });
  },
  reactShouldCreateDeleteOrgEventSource: (0, _reduxBundler.createSelector)('selectAuthToken', 'selectOrgEventSourceRaw', orgIdSelector, (token, raw, orgIdsToBeSubscribedTo) => {
    if (!token) return;

    for (const orgId of orgIdsToBeSubscribedTo) {
      if (!raw[orgId]) {
        return {
          actionCreator: 'doCreateOrgEventSource',
          args: [orgId]
        };
      }
    } // any to remove


    for (const key in raw) {
      if (!orgIdsToBeSubscribedTo.includes(key)) return {
        actionCreator: 'doRemoveOrgEventSource',
        args: [key]
      };
    }
  })
});

exports.default = _default;