"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getSessionBundle = exports.sessionResponseSnippet = void 0;

var _reduxBundler = require("redux-bundler");

var _milliseconds = _interopRequireDefault(require("milliseconds"));

var _toGqlQuery = require("../utils/to-gql-query");

var _generateBundle = require("../utils/generate-bundle");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectWithoutProperties(source, excluded) { if (source == null) return {}; var target = _objectWithoutPropertiesLoose(source, excluded); var key, i; if (Object.getOwnPropertySymbols) { var sourceSymbolKeys = Object.getOwnPropertySymbols(source); for (i = 0; i < sourceSymbolKeys.length; i++) { key = sourceSymbolKeys[i]; if (excluded.indexOf(key) >= 0) continue; if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue; target[key] = source[key]; } } return target; }

function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

const STARTED = 'SESSION_FETCH_STARTED';
const FAILED = 'SESSION_FETCH_FAILED';
const SESSION_RECEIVED = 'SESSION_RECEIVED';
const sessionResponseSnippet = (0, _toGqlQuery.toGqlQuery)(['multiFactorLast', 'multiFactorAuthExp', 'sessionRenewedAt', 'sessionExpiresAt', 'token', 'orgs{id roles status memberId}']);
/**
 * @typedef GetSessionBundleOptions
 * @property {number} [renewSessionWhenOlderThan=180000] Time in milliseconds
 *   after which to renew the token. Defaults to three minutes. Default is `180000`
 * @property {string[]} selectorsThatDetermineActivity If any of these return a
 *   truthy value, they will keep the session renewing.
 * @property {string} activeOrgIdSelectorName
 */

/**
 * @param {GetSessionBundleOptions} [opts]
 * @returns {ReduxBundlerBundle}
 */

exports.sessionResponseSnippet = sessionResponseSnippet;

const getSessionBundle = opts => {
  /** @type {GetSessionBundleOptions} */
  const options = _objectSpread({
    renewSessionWhenOlderThan: 180000
  }, opts);

  return (0, _generateBundle.generateBundle)({
    name: 'session',
    initialState: {
      token: '',
      loading: false,
      data: null,
      lastFetchError: null
    },
    actionTypes: {
      [STARTED]: {
        handler: state => _objectSpread(_objectSpread({}, state), {}, {
          loading: true
        })
      },
      [FAILED]: {
        handler: (state, payload) => {
          const {
            time
          } = payload;
          return _objectSpread(_objectSpread({}, state), {}, {
            loading: false,
            lastFetchError: time
          });
        }
      },
      [SESSION_RECEIVED]: {
        handler: (state, payload) => {
          const {
            token,
            data
          } = payload;
          return _objectSpread(_objectSpread({}, state), {}, {
            loading: false,
            data,
            token,
            lastFetchError: null
          });
        }
      }
    },
    selectSessionRaw: state => state.session,
    selectSessionData: state => state.session.data,
    selectSessionDataIsLoading: state => state.session.loading,
    selectSessionTimes: (0, _reduxBundler.createSelector)('selectSessionData', sessionData => {
      const keys = ['multiFactorAuthExp', 'multiFactorLast', 'sessionExpiresAt', 'sessionRenewedAt'];
      const data = sessionData || {};
      const result = {};

      for (const key of keys) {
        const asNumber = Number(data[key]);
        result[key] = isNaN(asNumber) ? null : asNumber;
      }

      return result;
    }),
    selectSessionHasValidMFA: (0, _reduxBundler.createSelector)('selectSessionTimes', 'selectAppTime', ({
      multiFactorAuthExp,
      multiFactorLast
    }, appTime) => !!multiFactorLast && multiFactorAuthExp > appTime),
    selectHasValidSession: (0, _reduxBundler.createSelector)('selectSessionTimes', 'selectAppTime', ({
      sessionExpiresAt,
      sessionRenewedAt
    }, appTime) => !!sessionRenewedAt && sessionExpiresAt > appTime),
    selectIsSignedIn: (0, _reduxBundler.createSelector)('selectHasAuthToken', 'selectHasValidSession', (authToken, hasValidSession) => !!(authToken && hasValidSession)),
    selectSessionOrgs: (0, _reduxBundler.createSelector)('selectSessionData', data => data && data.orgs || null),
    selectSessionOrgIds: (0, _reduxBundler.createSelector)('selectSessionOrgs', orgs => orgs && orgs.map(item => item.id) || []),
    selectActiveOrgMembership: (0, _reduxBundler.createSelector)('selectSessionOrgs', opts.activeOrgIdSelectorName, (orgs, activeOrgId) => orgs && orgs.find(org => org.id === activeOrgId)),
    selectActiveOrgMemberId: (0, _reduxBundler.createSelector)('selectActiveOrgMembership', membership => membership && membership.memberId),
    selectActiveOrgRoles: (0, _reduxBundler.createSelector)('selectActiveOrgMembership', membership => membership && membership.roles || []),
    selectIsOrgAdmin: (0, _reduxBundler.createSelector)('selectActiveOrgRoles', roles => roles.includes('admin')),
    doFetchSession: (update = false) => ({
      gqlFetch,
      dispatch,
      store
    }) => {
      dispatch({
        type: STARTED
      });
      let fetchPromise;

      if (update) {
        fetchPromise = gqlFetch("mutation extendSession($testOnly_overrideTime:String) {\n            extendSession(testOnly_overrideTime:$testOnly_overrideTime)".concat(sessionResponseSnippet, "\n          }"), {
          testOnly_overrideTime: window.Cypress ? Date.now().toString() : null
        }).then(res => res.data.extendSession);
      } else {
        fetchPromise = gqlFetch("{session".concat(sessionResponseSnippet, "}")).then(res => res.data.session);
      }

      return fetchPromise.then(res => {
        store.doReceiveSession(res);
      }).catch(err => {
        console.error(err);
        dispatch({
          type: FAILED,
          payload: {
            time: Date.now()
          }
        });
      });
    },
    doReceiveSession: updatedSession => ({
      store,
      dispatch
    }) => {
      const {
        token
      } = updatedSession,
            rest = _objectWithoutProperties(updatedSession, ["token"]);

      if (token !== store.selectAuthToken()) {
        store.doReceiveToken(token);
      }

      dispatch({
        type: SESSION_RECEIVED,
        payload: {
          token,
          data: rest
        }
      });
    },
    reactShouldExtendSession: (0, _reduxBundler.createSelector)('selectIsSignedIn', 'selectSessionData', 'selectSessionRaw', 'selectAppTime', ...options.selectorsThatDetermineActivity, (signedIn, data, raw, appTime, ...otherValues) => {
      const {
        sessionRenewedAt
      } = data || {};
      const {
        loading,
        lastExtendError
      } = raw;
      const active = otherValues.some(Boolean);

      if (!signedIn || !active || !sessionRenewedAt || loading) {
        return null;
      }

      const sessionAge = appTime - Number(sessionRenewedAt);

      if (lastExtendError) {
        const timeSince = appTime - lastExtendError; // take a break if the last one failed

        if (timeSince < _milliseconds.default.seconds(30)) {
          return false;
        }
      }

      if (sessionAge > options.renewSessionWhenOlderThan) {
        return {
          actionCreator: 'doFetchSession',
          args: [true]
        };
      }
    }),
    reactShouldFetchSession: (0, _reduxBundler.createSelector)('selectShowUserLoading', 'selectSessionRaw', 'selectAuthToken', (userLoading, raw, authToken) => {
      if (userLoading || raw.loading || !authToken || raw.lastError) {
        return;
      }

      if (raw.token !== authToken) {
        return {
          actionCreator: 'doFetchSession'
        };
      }
    })
  });
};

exports.getSessionBundle = getSessionBundle;