"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getFhirFetch = void 0;

var _handleHttpResponse = _interopRequireDefault(require("./handle-http-response"));

var _object = require("./object");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// @ts-check

/** @typedef {'NO_AUTH' | 'NO_USER'} BadAuthStatus */

/**
 * Get a configured fetcher for FHIR requests
 *
 * @param {{
 *   apiUrl: string
 *   getToken: () => string
 *   getOrgId: () => string
 *   handleNotAuthorized: () => any
 *   handleNoAuthentication: () => any
 * }} options
 * @returns {(pathname: string, opts?: { orgId?: string }) => Promise} Opts
 */
const getFhirFetch = options => {
  const {
    getToken,
    getOrgId,
    apiUrl,
    handleNoAuthentication,
    handleNotAuthorized
  } = options;
  return (relative, opts = {}) => {
    const [pathname, query] = relative.split('?');
    const tokenAddon = "token=".concat(getToken());
    const modifiedQueryString = query ? "".concat(query, "&").concat(tokenAddon) : tokenAddon;
    const orgId = opts.orgId || getOrgId();
    const fetchOptions = (0, _object.omit)(opts, ['orgId']);

    if (!orgId) {
      throw Error('NO_ORG_ID');
    }

    const base = "".concat(apiUrl, "/fhir/").concat(orgId);
    return fetch("".concat(base).concat(pathname, "?").concat(modifiedQueryString), fetchOptions).then(res => {
      (0, _handleHttpResponse.default)(res.status, {
        handleNoAuthentication,
        handleNotAuthorized
      });
      return res.json();
    });
  };
};

exports.getFhirFetch = getFhirFetch;